

function getAbsoluteBounds(entity)
    local b = entity.model.bounds
    local corners = {
        b.min,
        vec3(b.min.x, b.min.y, b.max.z),
        vec3(b.min.x, b.max.y, b.min.z),
        vec3(b.min.x, b.max.y, b.max.z),
        vec3(b.max.x, b.min.y, b.min.z),
        vec3(b.max.x, b.min.y, b.max.z),
        vec3(b.max.x, b.max.y, b.min.z),
        b.max
    }
    
    local transformedCorners = {}
    for _, corner in ipairs(corners) do
        table.insert(transformedCorners, entity:transformPoint(corner))
    end
    
    return transformedCorners
end

function drawAbsoluteBounds(entity)
    local transformedCorners = getAbsoluteBounds(entity)
    
    -- Connect the transformed corners with lines
    local lines = {
        {1, 2}, {2, 4}, {4, 3}, {3, 1},
        {5, 6}, {6, 8}, {8, 7}, {7, 5},
        {1, 5}, {2, 6}, {3, 7}, {4, 8}
    }
    
    for _, line in ipairs(lines) do
        scene.debug:line(transformedCorners[line[1]], transformedCorners[line[2]], color(255, 0, 0))
    end
end

function pointIsInBounds(point, modelEntity)
    -- Transform the point to the model's local space
    local localPoint = modelEntity:inverseTransformPoint(point)
    
    -- Get the model's original bounds
    local bounds = modelEntity.model.bounds
    
    return localPoint.x >= bounds.min.x and localPoint.x <= bounds.max.x and
    localPoint.y >= bounds.min.y and localPoint.y <= bounds.max.y and
    localPoint.z >= bounds.min.z and localPoint.z <= bounds.max.z
end

function absoluteBoundsIntersect(entityA, entityB)
    local cornersA = getAbsoluteBounds(entityA)
    local cornersB = getAbsoluteBounds(entityB)
    
    -- Get normals for entityA
    local normalsA = getBoxNormals(cornersA)
    
    -- Get normals for entityB
    local normalsB = getBoxNormals(cornersB)
    
    -- Check for separation on each normal
    for _, normal in ipairs(normalsA) do
        if isSeparatedOnAxis(normal, cornersA, cornersB) then
            return false
        end
    end
    
    for _, normal in ipairs(normalsB) do
        if isSeparatedOnAxis(normal, cornersA, cornersB) then
            return false
        end
    end
    
    return true
end

function getBoxNormals(corners)
    return {
        (corners[2] - corners[1]):normalize(),
        (corners[4] - corners[1]):normalize(),
        (corners[5] - corners[1]):normalize()
    }
end

function isSeparatedOnAxis(axis, cornersA, cornersB)
    local minA, maxA = projectToAxis(axis, cornersA)
    local minB, maxB = projectToAxis(axis, cornersB)
    
    return maxA < minB or maxB < minA
end

function projectToAxis(axis, corners)
    local min = math.huge
    local max = -math.huge
    
    for _, corner in ipairs(corners) do
        local projection = corner:dot(axis)
        min = math.min(min, projection)
        max = math.max(max, projection)
    end
    
    return min, max
end

