viewer.mode = STANDARD

function setup()
    scene = craft.scene()
    scene.sky.material.sky = color(72, 235, 215)       
    scene.sky.material.horizon = color(34, 30, 66)       
    scene.sky.material.ground = color(60, 74, 200)   
    
    -- Monkey
    monkey = scene:entity()
    monkey.model = craft.model(asset.builtin.Primitives.Monkey)
    monkey.material = craft.material(asset.builtin.Materials.Specular)
    monkey.material.diffuse = color(183, 48, 224)
    monkey.position = vec3(0, 0, 0)
    monkey.scale = vec3(1, 1, 1)
    monkey.rotation = quat.eulerAngles(0, 0, 0)
    
    -- Box
    box = scene:entity()
    box.model = craft.model(asset.builtin.Primitives.RoundedCube)
    box.material = craft.material(asset.builtin.Materials.Specular)
    box.material.diffuse = color(0, 0, 255)
    box.position = vec3(0, 0, 0)
    box.scale = vec3(0.04, 0.4, 0.25)
    box.rotation = quat.eulerAngles(0, 0, 0)
    
    -- Camera
    orbitViewer = scene.camera:add(OrbitViewer, monkey.position, 6, 1, 10)
    
    setUpParameters()
    print("Red lines are calculated boundary boxes.\n\nStretch and rotate monkey and boundary detection moves with it.\n\nMove box to see it turn green when boundaries overlap.")
end

-- Parameters for controlling the monkey's transformation
function setUpParameters()
    parameter.number("Monkey Scale X", 0.1, 3, 1)
    parameter.number("Monkey Scale Y", 0.1, 3, 1)
    parameter.number("Monkey Scale Z", 0.1, 3, 1)
    parameter.number("Monkey Rotate X", 0, 360, 12)
    parameter.number("Monkey Rotate Y", 0, 360, 212)
    parameter.number("Monkey Rotate Z", 0, 360, 0)
    
    -- Parameters for controlling the box's position
    parameter.number("Box Position X", -10, 10, 1.08)
    parameter.number("Box Position Y", -10, 10, 0.61)
    parameter.number("Box Position Z", -10, 10, -0.5)
end

function draw()
    --update scene
    scene:update(DeltaTime)
    scene:draw()
    
    --update entities
    monkey.scale = vec3(Monkey_Scale_X, Monkey_Scale_Y, Monkey_Scale_Z)
    monkey.rotation = quat.eulerAngles(Monkey_Rotate_X, Monkey_Rotate_Y, Monkey_Rotate_Z)
    box.position = vec3(Box_Position_X, Box_Position_Y, Box_Position_Z)
    
    -- Draw the absolute bounds
    drawAbsoluteBounds(monkey)
    drawAbsoluteBounds(box)
    
    -- Check if the box intersects the monkey's bounds
    if absoluteBoundsIntersect(monkey, box) then
        box.material.diffuse = color(0, 255, 0) -- Green if intersect
    else
        box.material.diffuse = color(255, 112, 0) -- Red if not
    end
end



